@@ -1,3 +1,52 @@
+#' Simulation from a time series model
+#' 
+#' Returns a time series based on the model object \code{object}.
+#' 
+#' With \code{simulate.Arima()}, the \code{object} should be produced by
+#' \code{\link{Arima}} or \code{\link{auto.arima}}, rather than
+#' \code{\link[stats]{arima}}. By default, the error series is assumed normally
+#' distributed and generated using \code{\link[stats]{rnorm}}. If \code{innov}
+#' is present, it is used instead. If \code{bootstrap=TRUE} and
+#' \code{innov=NULL}, the residuals are resampled instead.
+#' 
+#' When \code{future=TRUE}, the sample paths are conditional on the data. When
+#' \code{future=FALSE} and the model is stationary, the sample paths do not
+#' depend on the data at all. When \code{future=FALSE} and the model is
+#' non-stationary, the location of the sample paths is arbitrary, so they all
+#' start at the value of the first observation.
+#' 
+#' @param object An object of class "\code{ets}", "\code{Arima}", "\code{ar}"
+#' or "\code{nnetar}".
+#' @param nsim Number of periods for the simulated series. Ignored if either
+#' \code{xreg} or \code{innov} are not \code{NULL}.
+#' @param seed Either \code{NULL} or an integer that will be used in a call to
+#' \code{\link[base]{set.seed}} before simulating the time series. The default,
+#' \code{NULL}, will not change the random generator state.
+#' @param future Produce sample paths that are future to and conditional on the
+#' data in \code{object}. Otherwise simulate unconditionally.
+#' @param bootstrap Do simulation using resampled errors rather than normally
+#' distributed errors or errors provided as \code{innov}.
+#' @param innov A vector of innovations to use as the error series. Ignored if
+#' \code{bootstrap==TRUE}. If not \code{NULL}, the value of \code{nsim} is set
+#' to length of \code{innov}.
+#' @param xreg New values of \code{xreg} to be used for forecasting. The value
+#' of \code{nsim} is set to the number of rows of \code{xreg} if it is not
+#' \code{NULL}.
+#' @param lambda Box-Cox parameter. If not \code{NULL}, the simulated series is
+#' transformed using an inverse Box-Cox transformation with parameter
+#' \code{lamda}.
+#' @param ... Other arguments, not currently used.
+#' @return An object of class "\code{ts}".
+#' @author Rob J Hyndman
+#' @seealso \code{\link{ets}}, \code{\link{Arima}}, \code{\link{auto.arima}},
+#' \code{\link{ar}}, \code{\link{arfima}}, \code{\link{nnetar}}.
+#' @keywords ts
+#' @examples
+#' fit <- ets(USAccDeaths)
+#' plot(USAccDeaths, xlim=c(1973,1982))
+#' lines(simulate(fit, 36), col="red")
+#' 
+#' @export
 simulate.ets <- function(object, nsim=length(object$x), seed=NULL, future=TRUE, bootstrap=FALSE, innov=NULL, ...)
 {
   if(is.null(innov))
@@ -211,6 +260,8 @@ myarima.sim <- function (model, n, x, e, ...)
   return(x)
 }
 
+#' @rdname simulate.ets
+#' @export
 simulate.Arima <- function(object, nsim=length(object$x), seed=NULL, xreg=NULL, future=TRUE, bootstrap=FALSE, innov=NULL, lambda=object$lambda, ...)
 {
   #Error check:
@@ -353,8 +404,18 @@ simulate.Arima <- function(object, nsim=length(object$x), seed=NULL, xreg=NULL,
   }
   if (use.drift)
   {
-    dft <- as.matrix(1:nsim) + n
-    xreg <- cbind(dft,xreg)
+    # Remove existing drift column
+    if(NCOL(xreg)==1)
+      xreg <- NULL
+    else
+      xreg <- xreg[,!is.element(colnames(xreg),"drift"), drop=FALSE]
+    # Create new drift column for historical simulation
+      dft <- as.matrix(1:nsim)
+    # Adapt if future simulation
+    if(future)
+      dft <- dft + n
+    # Add to xreg
+    xreg <- cbind(drift=dft,xreg)
   }
   narma <- sum(object$arma[1L:4L])
   if(length(object$coef) > narma)
@@ -405,6 +466,8 @@ simulate.Arima <- function(object, nsim=length(object$x), seed=NULL, xreg=NULL,
   return(sim)
 }
 
+#' @rdname simulate.ets
+#' @export
 simulate.ar <- function(object, nsim=object$n.used, seed=NULL, future=TRUE, bootstrap=FALSE, innov=NULL, ...)
 {
   if(is.null(innov))
@@ -452,6 +515,8 @@ simulate.ar <- function(object, nsim=object$n.used, seed=NULL, future=TRUE, boot
     return(arima.sim(model,nsim,innov=e) + x.mean)
 }
 
+#' @rdname simulate.ets
+#' @export
 simulate.fracdiff <- function(object, nsim=object$n, seed=NULL, future=TRUE, bootstrap=FALSE, innov=NULL, ...)
 {
   x <- getResponse(object)
@@ -478,7 +543,8 @@ simulate.fracdiff <- function(object, nsim=object$n, seed=NULL, future=TRUE, boo
   return(unfracdiff(xx,ysim,n,nsim,object$d) + meanx)
 }
 
-
+#' @rdname simulate.ets
+#' @export
 simulate.nnetar <- function(object, nsim=length(object$x), seed=NULL, xreg=NULL, future=TRUE, bootstrap=FALSE, innov=NULL, lambda=object$lambda, ...)
 {
   if(is.null(innov))
