@@ -12,7 +12,7 @@ if(require(testthat))
     data <- datamat(mv_y,mv_x,v_y,v_x)
     expect_true(is.ts(data[,1]))
     expect_true(identical(tsp(data[,1]), tsp(data[,2])))
-    expect_true(NCOL(data)==4)
+    expect_true(NCOL(data)==8)
     expect_true(NCOL(data[,1])==2)
     expect_true("matrix" %in% class(data[,1]))
     expect_true(class(data)=="data.frame")
@@ -25,9 +25,4 @@ if(require(testthat))
     expect_true(class(data)=="data.frame")
     expect_true(!"data.frame"%in%class(data[,1]))
   })
-  
-  test_that("rename functions", {
-    data <- datamat(mv_y,mv_x,v_y,v_x,fourier(v_y,3), functions=TRUE)
-    expect_true(sum(grep("FN.fourier_", colnames(data)))>0)
-  })
 }
\ No newline at end of file
